package com.eksad.masterdata.controller.apidgi.dphlo;

import com.eksad.ddms.common.dto.GatewayDPHLODTO;
import com.eksad.ddms.common.dto.GatewayDPHLODetailDTO;
import com.eksad.ddms.common.h2.dto.ui.ResponseErrorDTO;
import com.eksad.ddms.common.h3.dto.dphlopull.DPHLOPullHeaderDTO;
import com.eksad.ddms.common.h3.dto.dphlopull.DPHLOPullParamDTO;
import com.eksad.ddms.common.h3.dto.dphlopull.ResponseDPHLOPullDTO;
import com.eksad.ddms.masterdata.h3.dto.dgi.DealerGroupMappingNewDTO;
import com.eksad.ddms.masterdata.h3.dto.dgi.DgiResponseDTO;
import com.eksad.masterdata.service.DGIService;
import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.text.ParseException;
import java.util.*;
import java.util.logging.Level;
import java.util.logging.Logger;

@RestController
@Slf4j
public class DGIDpHLORESTController {

    @Autowired
    DGIService dgiService;

    @RequestMapping(value = "/dgi-api/v1/dphlo/read",
            method = RequestMethod.POST,
            produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<Object> getDPHLO(@RequestHeader("GvtDealerId")String dealerId, @RequestBody DPHLOPullParamDTO dto) throws ParseException {
        try {
            System.out.println(new ObjectMapper().writeValueAsString(dto));
            dgiService.exceptionHelper(dto.getFromTime(),"yyyy-MM-dd HH:mm:ss","fromTime");
            dgiService.exceptionHelperNotMandatory(dto.getToTime(),"yyyy-MM-dd HH:mm:ss","toTime");

            if (dgiService.isNotValidDate(dto.getFromTime())){
                throw new Exception("format toTime tidak sesuai, seharusnya YYYY-MM-DD hh:mm:ss");
            } else if (dgiService.isNotValidDate(dto.getToTime())) {
                throw new Exception("format toTime tidak sesuai, seharusnya YYYY-MM-DD hh:mm:ss");
            }

            dgiService.checkToTimeIs7Days(dto.getFromTime(), dto.getToTime());
            dgiService.dealerValidation(dealerId,dto.getDealerId());
        } catch (Exception ex) {
            Logger.getLogger(DGIDpHLORESTController.class.getName()).log(Level.SEVERE, null, ex);
            ResponseErrorDTO res =  new ResponseErrorDTO();
            res.setMessage(ex.getMessage());
            res.setStatus(HttpStatus.BAD_REQUEST.value());
            res.setError(HttpStatus.BAD_REQUEST.getReasonPhrase());
            return ResponseEntity.badRequest().body(res);
        }
        return dgiService.postRequestDPHLO(dto);
    }

    @RequestMapping(value = "/dgi-api/v1/dphlo/add",
            method = RequestMethod.POST,
            produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<Object> postDPHLOApi(@RequestHeader("GvtDealerId")String dealerId,@RequestBody GatewayDPHLODTO dto) throws ParseException {
        try {
            System.out.println(new ObjectMapper().writeValueAsString(dto));
            dgiService.nullEmptyExceptionHelper(dto,new ArrayList<>());
            dgiService.exceptionHelper(dto.getTanggalPemesananHLO(),"dd/MM/yyyy","tanggalPemesananHLO");
            dgiService.exceptionHelper(dto.getCreatedTime(),"dd/MM/yyyy HH:mm:ss","createdTime");
            dgiService.exceptionHelper(dto.getModifiedTime(),"dd/MM/yyyy HH:mm:ss","modifiedTime");
            dgiService.dealerValidation(dealerId,dto.getDealerId());
            if (!dto.getParts().isEmpty() || dto.getParts() != null){
                for (GatewayDPHLODetailDTO parts: dto.getParts()){
                    dgiService.exceptionHelperNotMandatory(parts.getCreatedTime(),"dd/MM/yyyy HH:mm:ss","parts createdTime");
                    dgiService.exceptionHelperNotMandatory(parts.getModifiedTime(),"dd/MM/yyyy HH:mm:ss","parts modifiedTime");
                }
            }

        } catch (Exception ex) {
            Logger.getLogger(DGIDpHLORESTController.class.getName()).log(Level.SEVERE, null, ex);
            ResponseErrorDTO res =  new ResponseErrorDTO();
            res.setMessage(ex.getMessage());
            res.setStatus(HttpStatus.BAD_REQUEST.value());
            res.setError(HttpStatus.BAD_REQUEST.getReasonPhrase());
            return ResponseEntity.badRequest().body(res);
        }

        return dgiService.postRequestPullDPHLO(dto);

    }

    @RequestMapping(value = "/dgi-api/v1.3/dphlo/read",
            method = RequestMethod.POST,
            produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<Object> getDPHLO(@RequestHeader("GvtDealerId") String dealerId, @RequestBody DPHLOPullParamDTO dto, Boolean dgi) throws ParseException {
        try {
            System.out.println("disini payload dgi-api/v1.3/dphlo/read" + new ObjectMapper().writeValueAsString(dto));
            DgiResponseDTO dgiNew = dgiService.dgiv1_3Validator(dto.getDealerId(), dealerId);

            System.out.println("v1.3 dphlo/read");
            System.out.println(new ObjectMapper().writeValueAsString(dgiNew));
            if (dgiNew.getStatus().equals(0) || dgiNew.getStatus().equals("0")) {
                //return here
                return ResponseEntity.ok().body(dgiNew);
            }

            List<?> dataDgi = dgiNew.getData();

            List<DealerGroupMappingNewDTO> ob = Arrays.asList(new ObjectMapper().convertValue(dataDgi.toArray(), DealerGroupMappingNewDTO[].class));

            List<DPHLOPullHeaderDTO> data = new ArrayList<>();

            List<String> listDealer = new ArrayList<>();

            ob.forEach(x -> {
                listDealer.add(x.getDealerAhmCode());
            });

            for(String y : listDealer){
                dto.setDealerId(y);
                ResponseDPHLOPullDTO pullDPHLO = dgiService.postRequestDPHLO_v1_3(dto);
                System.out.println("v1.3/dphlo/read "+new ObjectMapper().writeValueAsString(pullDPHLO));
                if(pullDPHLO != null && pullDPHLO.getData() != null && !pullDPHLO.getData().isEmpty()) {
                    data.addAll(pullDPHLO.getData());
                }
            }

            if (data.isEmpty()) {
                dgiNew.setStatus(1);
                dgiNew.setData(null);
                dgiNew.putError("data tidak ditemukan");
                return ResponseEntity.ok(dgiNew);
            }

            Map<String, DPHLOPullHeaderDTO> map = new HashMap<>();
            data.forEach(x -> map.put(x.getIdHLODocument(), x));

            List<DPHLOPullHeaderDTO> listDataDistinct = new ArrayList<>(map.values());

            dgiNew.setData(listDataDistinct);

            return ResponseEntity.ok(dgiNew);
        } catch (Exception ex) {
            Logger.getLogger(DGIDpHLORESTController.class.getName()).log(Level.SEVERE, null, ex);
            ResponseErrorDTO res = new ResponseErrorDTO();
            res.setMessage(ex.getMessage());
//            res.setStatus(HttpStatus.BAD_REQUEST.value());
            res.setStatus(0);
            res.setError(HttpStatus.BAD_REQUEST.getReasonPhrase());
            return ResponseEntity.badRequest().body(res);
        }
    }

    @RequestMapping(value = "/dgi-api/v1.3/dphlo/add",
            method = RequestMethod.POST,
            produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<Object> postDPHLOApi(@RequestHeader("GvtDealerId") String dealerId, @RequestBody GatewayDPHLODTO dto, Boolean dgi) throws ParseException {
        try {
            dgiService.nullEmptyExceptionHelper(dto, new ArrayList<>());
            dgiService.exceptionHelper(dto.getTanggalPemesananHLO(), "dd/MM/yyyy", "tanggalPemesananHLO");
            dgiService.exceptionHelper(dto.getCreatedTime(), "dd/MM/yyyy HH:mm:ss", "createdTime");
            dgiService.exceptionHelper(dto.getModifiedTime(), "dd/MM/yyyy HH:mm:ss", "modifiedTime");
            if (!dto.getParts().isEmpty() || dto.getParts() != null) {
                for (GatewayDPHLODetailDTO parts : dto.getParts()) {
                    dgiService.exceptionHelperNotMandatory(parts.getCreatedTime(), "dd/MM/yyyy HH:mm:ss", "parts createdTime");
                    dgiService.exceptionHelperNotMandatory(parts.getModifiedTime(), "dd/MM/yyyy HH:mm:ss", "parts modifiedTime");
                }
            }

            DgiResponseDTO dgiNew = new DgiResponseDTO();

            if (dto.getDealerId() == null || dto.getDealerId().trim().equals("")) {
                dgiNew.setData(null);
                dgiNew.putError("dealerId Kosong");
                dgiNew.setStatus(0);
                return ResponseEntity.ok(dgiNew);
            }

            System.out.println("disini payload dgi-api/v1.3/dphlo/add" + new ObjectMapper().writeValueAsString(dto));
            dgiNew = dgiService.dgiv1_3Validator(dto.getDealerId(), dealerId);

            System.out.println("v1.3 dphlo/add");
            System.out.println(new ObjectMapper().writeValueAsString(dgiNew));
            if (dgiNew.getStatus().equals(0) || dgiNew.getStatus().equals("0")) {
                //return here
                return ResponseEntity.ok().body(dgiNew);
            }


            return dgiService.postRequestPullDPHLO(dto);

        } catch (Exception ex) {
            Logger.getLogger(DGIDpHLORESTController.class.getName()).log(Level.SEVERE, null, ex);
            ResponseErrorDTO res = new ResponseErrorDTO();
            res.setMessage(ex.getMessage());
            res.setStatus(HttpStatus.BAD_REQUEST.value());
            res.setError(HttpStatus.BAD_REQUEST.getReasonPhrase());
            return ResponseEntity.badRequest().body(res);
        }
    }
}
