package com.eksad.masterdata.controller.apidgi.invoicenjbnsc;

import com.eksad.ddms.common.dto.GatewayOrafinDTO;
import com.eksad.ddms.common.dto.GatewayPartOrafinDTO;
import com.eksad.ddms.common.dto.GatewayServiceOrafinDTO;
import com.eksad.masterdata.service.DGIService;
import com.eksad.ddms.common.h2.dto.apigateway.request.PullNotaJBSCRequestDTO;
import com.eksad.ddms.common.h2.dto.ui.ResponseErrorDTO;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import lombok.extern.slf4j.Slf4j;

import java.io.IOException;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

@RestController
@Slf4j
public class DGIInvoiceNjbNscRESTController {

    @Autowired
    DGIService dgiService;

    @RequestMapping(value = "/dgi-api/v1/inv2/read",
            method = RequestMethod.POST,
            produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<Object> pullNota(@RequestHeader("GvtDealerId") String dealerid, @RequestBody PullNotaJBSCRequestDTO dto) throws ParseException {
        try {
            System.out.println(new ObjectMapper().writeValueAsString(dto));
            dgiService.exceptionHelper(dto.getFromTime(), "yyyy-MM-dd HH:mm:ss", "fromtime");
            dgiService.exceptionHelperNotMandatory(dto.getToTime(), "yyyy-MM-dd HH:mm:ss", "totime");

            if (dgiService.isNotValidDate(dto.getFromTime())) {
                throw new Exception("format toTime tidak sesuai, seharusnya YYYY-MM-DD hh:mm:ss");
            } else if (dgiService.isNotValidDate(dto.getToTime())) {
                throw new Exception("format toTime tidak sesuai, seharusnya YYYY-MM-DD hh:mm:ss");
            }

            dgiService.checkToTimeIs7Days(dto.getFromTime(), dto.getToTime());

            dgiService.dealerValidation(dealerid, dto.getDealerId());
        } catch (Exception ex) {
            Logger.getLogger(DGIInvoiceNjbNscRESTController.class.getName()).log(Level.SEVERE, null, ex);
            ResponseErrorDTO res = new ResponseErrorDTO();
            res.setMessage(ex.getMessage());
            res.setStatus(HttpStatus.BAD_REQUEST.value());
            res.setError(HttpStatus.BAD_REQUEST.getReasonPhrase());
            return ResponseEntity.badRequest().body(res);
        }
        return dgiService.postRequestPullNJBNSC(dto);
    }

    @RequestMapping(value = "/dgi-api/v1/inv2/add",
            method = RequestMethod.POST,
            consumes = MediaType.APPLICATION_JSON_VALUE,
            produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<Object> postWorkOrderOrafin(@RequestHeader("GvtDealerId") String dealerid, @RequestBody GatewayOrafinDTO dto) throws IOException {
        try {
            System.out.println(new ObjectMapper().writeValueAsString(dto));
            List<String> pass = new ArrayList<>();
            if (dto.getNoWorkOrder().contains("WO")) {
                //kalo wo njb wajib
                if (dto.getNoNJB() == null || dto.getNoNJB().isEmpty()) {
                    throw new Exception("No NJB Tidak Boleh Kosong, Mohon Periksa Kembali Data");
                }
                if (dto.getTanggalNJB() == null || dto.getTanggalNJB().isEmpty()) {
                    throw new Exception("Tanggal NJB Tidak Boleh Kosong, Mohon Periksa Kembali Data");
                }
                if (dto.getTotalHargaNJB() == null || dto.getTotalHargaNJB() == 0 || dto.getTotalHargaNJB().isNaN()) {
                    throw new Exception("Total Harga NJB Tidak Boleh Kosong, Mohon Periksa Kembali Data");
                }
                if (dto.getHondaIdSA() == null || dto.getHondaIdSA().isEmpty()) {
                    throw new Exception("Honda Id SA Tidak Boleh Kosong, Mohon Periksa Kembali Data");
                }
                if (dto.getHondaIdMekanik() == null || dto.getHondaIdMekanik().isEmpty()) {
                    throw new Exception("Honda Id Mekanik Tidak Boleh Kosong, Mohon Periksa Kembali Data");
                }
                if (dto.getNjb().isEmpty() || dto.getNjb() == null) {
                    throw new Exception("NJB Tidak Boleh Kosong, Mohon Periksa Kembali Data");
                }

                if (dto.getNoNSC() == null || dto.getNoNSC().isEmpty() && !dto.getTanggalNSC().isEmpty()) {
                    throw new Exception("No NSC harus dilengkapi");
                } else if (dto.getNoNSC() == null || dto.getNoNSC().isEmpty() && dto.getTotalHargaNSC() != 0) {
                    throw new Exception("NO NSC harus dilengkapi");
                } else if (dto.getNoNSC() == null || dto.getNoNSC().isEmpty() && !dto.getNsc().isEmpty()) {
                    throw new Exception("No NSC harus dilengkapi");
                } else if (!dto.getNoNSC().isEmpty() && dto.getTanggalNSC().isEmpty()) {
                    throw new Exception("Tanggal NSC harus dilengkapi");
                } else if (!dto.getNoNSC().isEmpty() && dto.getTotalHargaNSC() == 0) {
                    throw new Exception("Total Harga NSC harus dilengkapi");
                } else if (!dto.getNoNSC().isEmpty() && dto.getNsc().isEmpty()) {
                    throw new Exception("NSC harus dilengkapi");
                }
            }
            if (dto.getNoWorkOrder().contains("SO")) {
                //kalo so nsc wajib
                if (dto.getNoNSC() == null || dto.getNoNSC().isEmpty()) {
                    throw new Exception("No NSC Tidak Boleh Kosong, Mohon Periksa Kembali Data");
                }
                if (dto.getTanggalNSC() == null || dto.getTanggalNSC().isEmpty()) {
                    throw new Exception("Tanggal NSC Tidak Boleh Kosong, Mohon Periksa Kembali Data");
                }
                if (dto.getTotalHargaNSC() == null || dto.getTotalHargaNSC() == 0 || dto.getTotalHargaNSC().isNaN()) {
                    throw new Exception("Total Harga Tidak Boleh Kosong, Mohon Periksa Kembali Data");
                }

                if (dto.getNoNJB() == null || dto.getNoNJB().isEmpty() && !dto.getTanggalNJB().isEmpty()) {
                    throw new Exception("No NJB harus dilengkapi");
                } else if (dto.getNoNJB() == null || dto.getNoNJB().isEmpty() && dto.getTotalHargaNJB() != 0) {
                    throw new Exception("No NJB harus dilengkapi");
                } else if (dto.getNoNJB() == null || dto.getNoNJB().isEmpty() && !dto.getHondaIdMekanik().isEmpty()) {
                    throw new Exception("No NJB harus dilengkapi");
                } else if (dto.getNoNJB() == null || dto.getNoNJB().isEmpty() && !dto.getHondaIdSA().isEmpty()) {
                    throw new Exception("No NJB harus dilengkapi");
                } else if (dto.getNoNJB() == null || dto.getNoNJB().isEmpty() && !dto.getNjb().isEmpty()) {
                    throw new Exception("No NJB harus dilengkapi");
                } else if (!dto.getNoNJB().isEmpty() && dto.getTanggalNJB().isEmpty()) {
                    throw new Exception("Tanggal NJB harus dilengkapi");
                } else if (!dto.getNoNJB().isEmpty() && dto.getTotalHargaNJB() == 0) {
                    throw new Exception("Total Harga NJB harus dilengkapi");
                } else if (!dto.getNoNJB().isEmpty() && dto.getHondaIdMekanik().isEmpty()) {
                    throw new Exception("Honda Id Mekanik harus dilengkapi");
                } else if (!dto.getNoNJB().isEmpty() && dto.getHondaIdSA().isEmpty()) {
                    throw new Exception("Honda Id SA harus dilengkapi");
                } else if (!dto.getNoNJB().isEmpty() && dto.getNjb().isEmpty()) {
                    throw new Exception("NJB harus dilengkapi");
                }
            }
//        nullEmptyExceptionHelper(dto,pass);
            dgiService.exceptionHelperNotMandatory(dto.getCreatedTime(), "dd/MM/yyyy HH:mm:ss", "createdTime");
            dgiService.exceptionHelperNotMandatory(dto.getModifiedTime(), "dd/MM/yyyy HH:mm:ss", "modifiedTime");
            dgiService.exceptionHelperNotMandatory(dto.getTanggalNJB(), "dd/MM/yyyy HH:mm:ss", "tanggalNJB");
            dgiService.exceptionHelperNotMandatory(dto.getTanggalNSC(), "dd/MM/yyyy HH:mm:ss", "tanggalNSC");
            dgiService.dealerValidation(dealerid, dto.getDealerId());
            if (!dto.getNjb().isEmpty() || dto.getNjb() != null) {
                for (GatewayServiceOrafinDTO njb : dto.getNjb()) {
                    dgiService.exceptionHelperNotMandatory(njb.getCreatedTime(), "dd/MM/yyyy HH:mm:ss", "njb createdTime");
                    dgiService.exceptionHelperNotMandatory(njb.getModifiedTime(), "dd/MM/yyyy HH:mm:ss", "njb modifiedTime");
                }
            }
            if (!dto.getNsc().isEmpty() || dto.getNsc() != null) {
                for (GatewayPartOrafinDTO nsc : dto.getNsc()) {
                    dgiService.exceptionHelperNotMandatory(nsc.getCreatedTime(), "dd/MM/yyyy HH:mm:ss", "nsc createdTime");
                    dgiService.exceptionHelperNotMandatory(nsc.getModifiedTime(), "dd/MM/yyyy HH:mm:ss", "nsc modifiedTime");
                }
            }
        } catch (Exception ex) {
            Logger.getLogger(DGIInvoiceNjbNscRESTController.class.getName()).log(Level.SEVERE, null, ex);
            ResponseErrorDTO res = new ResponseErrorDTO();
            res.setMessage(ex.getMessage());
            res.setStatus(HttpStatus.BAD_REQUEST.value());
            res.setError(HttpStatus.BAD_REQUEST.getReasonPhrase());
            return ResponseEntity.badRequest().body(res);
        }
        return dgiService.postWorkOrderOrafin(dto);
    }

}
