package com.eksad.masterdata.controller.apidgi.managepartsales;

import com.eksad.ddms.common.h2.dto.ui.ResponseErrorDTO;
import com.eksad.ddms.common.h3.dto.managepartsales.HeaderSOManagePartSalesDTO;
import com.eksad.ddms.common.h3.dto.managepartsales.ParamManageSalesDTO;
import com.eksad.ddms.common.h3.dto.managepartsales.ResponseManagePartSalesDTO;
import com.eksad.ddms.masterdata.h3.dto.dgi.DealerGroupMappingNewDTO;
import com.eksad.ddms.masterdata.h3.dto.dgi.DgiResponseDTO;
import com.eksad.masterdata.service.DGIService;
import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.text.ParseException;
import java.util.*;
import java.util.logging.Level;
import java.util.logging.Logger;

@RestController
@Slf4j
public class DGIManagePartSalesRESTController {
    @Autowired
    DGIService dgiService;

    @RequestMapping(value = "/dgi-api/v1/prsl/read",
            method = RequestMethod.POST,
            produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<Object> getSO(@RequestHeader("GvtDealerId")String dealerId, @RequestBody ParamManageSalesDTO dto) throws ParseException {
        try {
            System.out.println(new ObjectMapper().writeValueAsString(dto));
            dgiService.exceptionHelper(dto.getFromTime(),"yyyy-MM-dd HH:mm:ss","fromTime");
            dgiService.exceptionHelperNotMandatory(dto.getToTime(),"yyyy-MM-dd HH:mm:ss","toTime");

            if (dgiService.isNotValidDate(dto.getFromTime())){
                throw new Exception("format toTime tidak sesuai, seharusnya YYYY-MM-DD hh:mm:ss");
            } else if (dgiService.isNotValidDate(dto.getToTime())){
                throw new Exception("format toTime tidak sesuai, seharusnya YYYY-MM-DD hh:mm:ss");
            }

            dgiService.checkToTimeIs7Days(dto.getFromTime(), dto.getToTime());

            dgiService.dealerValidation(dealerId,dto.getDealerId());
        } catch (Exception ex) {
            Logger.getLogger(DGIManagePartSalesRESTController.class.getName()).log(Level.SEVERE, null, ex);
            ResponseErrorDTO res =  new ResponseErrorDTO();
            res.setMessage(ex.getMessage());
            res.setStatus(HttpStatus.BAD_REQUEST.value());
            res.setError(HttpStatus.BAD_REQUEST.getReasonPhrase());
            return ResponseEntity.badRequest().body(res);
        }
        return dgiService.postRequestPullManagePartSales(dto);
    }

    @RequestMapping(value = "/dgi-api/v1.3/prsl/read",
            method = RequestMethod.POST,
            produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<Object> getSO(@RequestHeader("GvtDealerId") String dealerId, @RequestBody ParamManageSalesDTO dto, Boolean dgi) throws ParseException {
        try {
//            if(StringUtil.hasValue(dto.getDealerId()) && StringUtil.hasValue(dto.getNoSO())){
            System.out.println(new ObjectMapper().writeValueAsString(dto));
            dgiService.exceptionHelper(dto.getFromTime(), "yyyy-MM-dd HH:mm:ss", "fromTime");
            dgiService.exceptionHelperNotMandatory(dto.getToTime(), "yyyy-MM-dd HH:mm:ss", "toTime");

            if (dgiService.isNotValidDate(dto.getFromTime())) {
                throw new Exception("format toTime tidak sesuai, seharusnya YYYY-MM-DD hh:mm:ss");
            } else if (dgiService.isNotValidDate(dto.getToTime())) {
                throw new Exception("format toTime tidak sesuai, seharusnya YYYY-MM-DD hh:mm:ss");
            }
            dgiService.checkToTimeIs7Days(dto.getFromTime(), dto.getToTime());

            System.out.println("disini payload dgi-api/v1.3/prsl/read" + new ObjectMapper().writeValueAsString(dto));
            DgiResponseDTO dgiNew = dgiService.dgiv1_3Validator(dto.getDealerId(), dealerId);

            System.out.println("v1.3 prsl/read");
            System.out.println(new ObjectMapper().writeValueAsString(dgiNew));
            if (dgiNew.getStatus().equals(0) || dgiNew.getStatus().equals("0")) {
                //return here
                return ResponseEntity.ok().body(dgiNew);
            }

            List<?> dataDgi = dgiNew.getData();

            List<DealerGroupMappingNewDTO> ob = Arrays.asList(new ObjectMapper().convertValue(dataDgi.toArray(), DealerGroupMappingNewDTO[].class));

            List<HeaderSOManagePartSalesDTO> data = new ArrayList<>();

            List<String> listDealer = new ArrayList<>();

            ob.forEach(x -> {
                listDealer.add(x.getDealerAhmCode());
            });

            for(String y: listDealer){
                dto.setDealerId(y);
                ResponseManagePartSalesDTO pullManagePartSales = new ResponseManagePartSalesDTO();
                pullManagePartSales = dgiService.postRequestPullManagePartSales_v1_3(dto);

                System.out.println("v1.3/prsl/read "+new ObjectMapper().writeValueAsString(pullManagePartSales));
                if(pullManagePartSales != null && pullManagePartSales.getData() != null && !pullManagePartSales.getData().isEmpty()) {
                    data.addAll(pullManagePartSales.getData());
                }
            }
            if (data.isEmpty()) {
                dgiNew.setStatus(1);
                dgiNew.setData(null);
                dgiNew.putError("data tidak ditemukan");
                return ResponseEntity.ok(dgiNew);
            }

            Map<String, HeaderSOManagePartSalesDTO> map = new HashMap<>();
            data.forEach(x -> map.put(x.getNoSO(), x));
            List<HeaderSOManagePartSalesDTO> listDataDistinct = new ArrayList<>(map.values());

            dgiNew.setData(listDataDistinct);

            return ResponseEntity.ok(dgiNew);
        } catch (Exception ex) {
            Logger.getLogger(DGIManagePartSalesRESTController.class.getName()).log(Level.SEVERE, null, ex);
            ResponseErrorDTO res = new ResponseErrorDTO();
            res.setMessage(ex.getMessage());
            res.setStatus(0);
            res.setError(HttpStatus.BAD_REQUEST.getReasonPhrase());
            return ResponseEntity.badRequest().body(res);
        }
    }
}
