package com.eksad.masterdata.controller.apidgi.partinbound;

import com.eksad.ddms.common.h2.dto.ui.ResponseErrorDTO;
import com.eksad.ddms.common.h3.dto.managepartinbounds.PartsManageInboundParamDTO;
import com.eksad.ddms.common.h3.dto.managepartinbounds.ReceivingManagePartsInboundDTO;
import com.eksad.ddms.common.h3.dto.managepartinbounds.ResponseManagePartsInboundDTO;
import com.eksad.ddms.masterdata.h3.dto.dgi.DealerGroupMappingNewDTO;
import com.eksad.ddms.masterdata.h3.dto.dgi.DgiResponseDTO;
import com.eksad.masterdata.service.DGIService;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.text.ParseException;
import java.util.*;
import java.util.logging.Level;
import java.util.logging.Logger;

@RestController
@Slf4j
public class DGIPartInboundRESTController {
    @Autowired
    DGIService dgiService;

    @RequestMapping(value = "/dgi-api/v1/pinb/read",
            method = RequestMethod.POST,
            produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<Object> getPartManageInbound(@RequestHeader("GvtDealerId") String dealerId, @RequestBody PartsManageInboundParamDTO dto) throws ParseException, JsonProcessingException {
        try {
            System.out.println(new ObjectMapper().writeValueAsString(dto));
            dgiService.exceptionHelper(dto.getFromTime(), "yyyy-MM-dd HH:mm:ss", "fromTime");
            dgiService.exceptionHelperNotMandatory(dto.getToTime(), "yyyy-MM-dd HH:mm:ss", "toTime");

            if (dgiService.isNotValidDate(dto.getFromTime())) {
                throw new Exception("format toTime tidak sesuai, seharusnya YYYY-MM-DD hh:mm:ss");
            } else if (dgiService.isNotValidDate(dto.getToTime())) {
                throw new Exception("format toTime tidak sesuai, seharusnya YYYY-MM-DD hh:mm:ss");
            }

            dgiService.checkToTimeIs7Days(dto.getFromTime(), dto.getToTime());
            dgiService.dealerValidation(dealerId, dto.getDealerId());
        } catch (Exception ex) {
            Logger.getLogger(DGIPartInboundRESTController.class.getName()).log(Level.SEVERE, null, ex);
            ResponseErrorDTO res = new ResponseErrorDTO();
            res.setMessage(ex.getMessage());
            res.setStatus(HttpStatus.BAD_REQUEST.value());
            res.setError(HttpStatus.BAD_REQUEST.getReasonPhrase());
            return ResponseEntity.badRequest().body(res);
        }
        return dgiService.postRequestPullManagePartInbound(dto);
//        return null;
    }

    @RequestMapping(value = "/dgi-api/v1.3/pinb/read",
            method = RequestMethod.POST,
            produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<Object> getPartManageInbound(@RequestHeader("GvtDealerId") String dealerId, @RequestBody PartsManageInboundParamDTO dto, Boolean dgi) throws ParseException, JsonProcessingException {
        try {
            dgiService.exceptionHelper(dto.getFromTime(), "yyyy-MM-dd HH:mm:ss", "fromTime");
            dgiService.exceptionHelperNotMandatory(dto.getToTime(), "yyyy-MM-dd HH:mm:ss", "toTime");

            if (dgiService.isNotValidDate(dto.getFromTime())) {
                throw new Exception("format toTime tidak sesuai, seharusnya YYYY-MM-DD hh:mm:ss");
            } else if (dgiService.isNotValidDate(dto.getToTime())) {
                throw new Exception("format toTime tidak sesuai, seharusnya YYYY-MM-DD hh:mm:ss");
            }

            dgiService.checkToTimeIs7Days(dto.getFromTime(), dto.getToTime());

            System.out.println("disini payload dgi-api/v1.3/pinb/read" + new ObjectMapper().writeValueAsString(dto));
            DgiResponseDTO dgiNew = dgiService.dgiv1_3Validator(dto.getDealerId(), dealerId);

            System.out.println("v1.3 pinb/read");
            System.out.println(new ObjectMapper().writeValueAsString(dgiNew));
            if (dgiNew.getStatus().equals(0) || dgiNew.getStatus().equals("0")) {
                //return here
                return ResponseEntity.ok().body(dgiNew);
            }

            List<?> dataDgi = dgiNew.getData();

            List<DealerGroupMappingNewDTO> ob = Arrays.asList(new ObjectMapper().convertValue(dataDgi.toArray(), DealerGroupMappingNewDTO[].class));

            List<ReceivingManagePartsInboundDTO> data = new ArrayList<>();

            System.out.println("ddd" + new ObjectMapper().writeValueAsString(ob));

            List<String> dealerList = new ArrayList<>();
            ob.forEach(x -> {
                dealerList.add(x.getDealerAhmCode());
            });

            System.out.println("dealerList "+ new ObjectMapper().writeValueAsString(dealerList));

            for (String y : dealerList) {
                dto.setDealerId(y);
                ResponseManagePartsInboundDTO pullManagePartInbound = dgiService.postRequestPullManagePartInbound_v1_3(dto);
                if(pullManagePartInbound != null || pullManagePartInbound.getData() != null || !pullManagePartInbound.getData().isEmpty()){
                    data.addAll(pullManagePartInbound.getData());
                }
            }

            if (data.isEmpty()) {
                dgiNew.setStatus(1);
                dgiNew.setData(null);
                dgiNew.putError("data tidak ditemukan");
                return ResponseEntity.ok(dgiNew);
            }

            Map<String, ReceivingManagePartsInboundDTO> map = new HashMap<>();
            data.forEach(x -> map.put(x.getNoPenerimaan(), x));
            List<ReceivingManagePartsInboundDTO> listDataDistinct = new ArrayList<>(map.values());

            dgiNew.setData(listDataDistinct);

            return ResponseEntity.ok(dgiNew);
        } catch (Exception ex) {
            Logger.getLogger(DGIPartInboundRESTController.class.getName()).log(Level.SEVERE, null, ex);
            ResponseErrorDTO res = new ResponseErrorDTO();
            res.setMessage(ex.getMessage());
//            res.setStatus(HttpStatus.BAD_REQUEST.value());
            res.setStatus(0);
            res.setError(HttpStatus.BAD_REQUEST.getReasonPhrase());
            return ResponseEntity.badRequest().body(res);
        }
    }
}
