package com.eksad.masterdata.controller.apidgi.partinvoice;

import com.eksad.ddms.masterdata.h3.dto.dgi.*;
import com.eksad.masterdata.service.DGIService;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.time.format.DateTimeParseException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.LinkedHashMap;
import java.util.List;

import static java.time.temporal.ChronoUnit.DAYS;

@RestController
@Slf4j
public class DGIPartInvoiceRESTController {

    @Autowired
    DGIService dgiService;

    @RequestMapping(value = "/dgi-api/v1.3/mdinvh3/read", method = RequestMethod.POST)
    public ResponseEntity<?> getPartsInvoice(@RequestHeader("GvtDealerId") String dealerID,
                                             @RequestBody PartsInvoiceRequestDTO request, Boolean dgi) throws Exception {
        PartsInvoiceResponseDTO response = new PartsInvoiceResponseDTO();
        DateTimeFormatter dateTimeFormatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
        LinkedHashMap<String, String> errorList = new LinkedHashMap<>();

        Integer errorNumber = 0;

        DgiResponseDTO dgiNew = new DgiResponseDTO();

        System.out.println("disini payload dgi-api/v1.3/mdinvh3/read " + new ObjectMapper().writeValueAsString(request));
        dgiNew = dgiService.dgiv1_3Validator(request.getDealerId(), dealerID);

        System.out.println("v1.3 mdinvh3/read");
        System.out.println(new ObjectMapper().writeValueAsString(dgiNew));
        if (dgiNew.getStatus().equals(0) || dgiNew.getStatus().equals("0")) {
            //return here
            return ResponseEntity.ok().body(dgiNew);
        }

        List<String> groupDealer = new ArrayList<>();

        List<?> dataDgi = dgiNew.getData();

        List<DealerGroupMappingNewDTO> ob = Arrays.asList(new ObjectMapper().convertValue(dataDgi.toArray(), DealerGroupMappingNewDTO[].class));

        ob.forEach(x -> {
            groupDealer.add(x.getDealerAhmCode());
        });

        //Validasi fromTime
        if (request.getFromTime() == null) {
            errorNumber++;
            errorList.put("message" + errorNumber, "fromTime tidak boleh kosong");
        } else {
            String fromTime = request.getFromTime();
            try {
                LocalDateTime localFromTime = LocalDateTime.parse(fromTime, dateTimeFormatter);
            } catch (DateTimeParseException dte) {
                errorNumber++;
                errorList.put("message" + errorNumber, "Format fromTime salah. Seharusnya yyyy-MM-dd HH:mm:ss");
            }
        }

        //Validasi toTime
        if (request.getToTime() == null) {
            LocalDateTime currTime = LocalDateTime.now();
            String formattedDate = currTime.format(dateTimeFormatter);
            request.setToTime(formattedDate);
        } else {
            String toTime = request.getToTime();
            try {
                LocalDateTime localFromTime = LocalDateTime.parse(toTime, dateTimeFormatter);
            } catch (DateTimeParseException dte) {
                errorNumber++;
                errorList.put("message" + errorNumber, "Format toTime salah. Seharusnya yyyy-MM-dd HH:mm:ss");
            }
        }

        //Validasi range
        if (request.getFromTime() != null && request.getToTime() != null) {
            try {
                LocalDateTime localfTime = LocalDateTime.parse(request.getFromTime(), dateTimeFormatter);
                LocalDateTime localtTime = LocalDateTime.parse(request.getToTime(), dateTimeFormatter);

                Long daysBetween = DAYS.between(localfTime, localtTime);
                if (daysBetween > 7) {
                    errorNumber++;
                    errorList.put("message" + errorNumber, "Range hari tidak boleh lebih dari 7 hari");
                }

                if (localfTime.isAfter(localtTime)) {
                    errorNumber++;
                    errorList.put("message" + errorNumber, "toTime tidak boleh sebelum fromTime");
                }
            } catch (DateTimeParseException dte) {

            }
        }

        if (errorNumber > 0) {
            dgiNew.setStatus(0);
            dgiNew.setMessage(errorList);
            dgiNew.setData(null);
            return ResponseEntity.ok(dgiNew);
        }

        List<PartsInvoiceDataDTO> dataRes = new ArrayList<>();

        for (String y : groupDealer) {
            request.setDealerId(y);
            request.setNoPO(request.getNoPO()==null ? "" : request.getNoPO());
            PartsInvoiceResponseDTO dd = dgiService.postPortalDamParts(request);
            try {
                System.out.println("dimarih "+new ObjectMapper().writeValueAsString(dd));
            } catch (JsonProcessingException e) {
                throw new RuntimeException(e);
            }
            if(dd != null && dd.getData() != null ) {
                if(!dd.getData().isEmpty()) {
                    dataRes.addAll(dd.getData());
                }
            }
        }

        dgiNew.setStatus(1);
        dgiNew.setMessage(null);
        if(dataRes.isEmpty()){
            dgiNew.setData(null);
            dgiNew.putError("data tidak ditemukan");
        }else{
            dgiNew.setData(dataRes);
        }

        return ResponseEntity.ok(dgiNew);
    }
}
