package com.eksad.masterdata.controller.apidgi.unpaidhlo;

import com.eksad.ddms.common.h2.dto.apigateway.UnpaidDPHLOPullHeaderV2ResponseDTO;
import com.eksad.ddms.common.h2.dto.apigateway.UnpaidDPHLOPullParamDTO;
import com.eksad.ddms.common.h2.dto.apigateway.UnpaidDPHLOPullResponseDTO;
import com.eksad.ddms.common.h2.dto.ui.ResponseErrorDTO;
import com.eksad.ddms.masterdata.h3.dto.dgi.DealerGroupMappingNewDTO;
import com.eksad.ddms.masterdata.h3.dto.dgi.DgiResponseDTO;
import com.eksad.masterdata.service.DGIService;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestHeader;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;

import java.text.ParseException;
import java.util.*;
import java.util.logging.Level;
import java.util.logging.Logger;

public class DGIUnpaidHLORESTController {

    @Autowired
    DGIService dgiService;

    @RequestMapping(value = "/dgi-api/v1/unpaidhlo/read",
            method = RequestMethod.POST,
            produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<?> UnpaidDPHLOApigateway(@RequestHeader("GvtDealerId")String dealerId, @RequestBody UnpaidDPHLOPullParamDTO dto) throws ParseException {
        try {
            System.out.println(new ObjectMapper().writeValueAsString(dto));
            dgiService.exceptionHelper(dto.getFromTime(),"yyyy-MM-dd HH:mm:ss","fromTime");
            dgiService.exceptionHelperNotMandatory(dto.getToTime(),"yyyy-MM-dd HH:mm:ss","toTime");

            if (dgiService.isNotValidDate(dto.getFromTime())){
                throw new Exception("format toTime tidak sesuai, seharusnya YYYY-MM-DD hh:mm:ss");
            }

            else if (dgiService.isNotValidDate(dto.getToTime())){
                throw new Exception("format toTime tidak sesuai, seharusnya YYYY-MM-DD hh:mm:ss");
            }

            dgiService.checkToTimeIs7Days(dto.getFromTime(), dto.getToTime());
            dgiService.dealerValidation(dealerId,dto.getDealerId());
        } catch (Exception ex) {
            Logger.getLogger(DGIUnpaidHLORESTController.class.getName()).log(Level.SEVERE, null, ex);
            ResponseErrorDTO res =  new ResponseErrorDTO();
            res.setMessage(ex.getMessage());
            res.setStatus(HttpStatus.BAD_REQUEST.value());
            res.setError(HttpStatus.BAD_REQUEST.getReasonPhrase());
            return ResponseEntity.badRequest().body(res);
        }
        return dgiService.postRequestPullUnpaidHLO(dto);
    }

    @RequestMapping(value = "/dgi-api/v1.3/unpaidhlo/read",
            method = RequestMethod.POST,
            produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<?> UnpaidDPHLOApigatewayIn(@RequestHeader("GvtDealerId")String dealerId, @RequestBody UnpaidDPHLOPullParamDTO dto) throws ParseException {
        try {
            System.out.println(new ObjectMapper().writeValueAsString(dto));
            dgiService.exceptionHelper(dto.getFromTime(),"yyyy-MM-dd HH:mm:ss","fromTime");
            dgiService.exceptionHelperNotMandatory(dto.getToTime(),"yyyy-MM-dd HH:mm:ss","toTime");

            if (dgiService.isNotValidDate(dto.getFromTime())){
                throw new Exception("format toTime tidak sesuai, seharusnya YYYY-MM-DD hh:mm:ss");
            }

            else if (dgiService.isNotValidDate(dto.getToTime())){
                throw new Exception("format toTime tidak sesuai, seharusnya YYYY-MM-DD hh:mm:ss");
            }

            dgiService.checkToTimeIs7Days(dto.getFromTime(), dto.getToTime());
            System.out.println("disini payload dgi-api/v1.3/unpaidhlo/read" + new ObjectMapper().writeValueAsString(dto));
            DgiResponseDTO dgiNew = dgiService.dgiv1_3Validator(dto.getDealerId(), dealerId);

            System.out.println("v1.3 unpaidhlo/read");
            System.out.println(new ObjectMapper().writeValueAsString(dgiNew));
            if (dgiNew.getStatus().equals(0) || dgiNew.getStatus().equals("0")) {
                //return here
                return ResponseEntity.ok().body(dgiNew);
            }

            List<?> dataDgi = dgiNew.getData();

            List<DealerGroupMappingNewDTO> ob = Arrays.asList(new ObjectMapper().convertValue(dataDgi.toArray(), DealerGroupMappingNewDTO[].class));

            List<String> dealerList = new ArrayList<>();
            ob.forEach(x -> {
                dealerList.add(x.getDealerAhmCode());
            });

            System.out.println("dealerList "+ new ObjectMapper().writeValueAsString(dealerList));

            UnpaidDPHLOPullHeaderV2ResponseDTO res = new UnpaidDPHLOPullHeaderV2ResponseDTO();
            List<UnpaidDPHLOPullResponseDTO> listData = new ArrayList<>();

            for (String y : dealerList) {
                dto.setDealerId(y);
                List<UnpaidDPHLOPullResponseDTO> listRes = dgiService.postRequestPullUnpaidHLO(dto).getBody().getData();
                listData.addAll(listRes != null ? listRes : new ArrayList<>());
            }

            Map<String, UnpaidDPHLOPullResponseDTO> map = new HashMap<>();
            listData.forEach(x -> map.put(x.getIdHLODocument(), x));
            List<UnpaidDPHLOPullResponseDTO> listDataDistinct = new ArrayList<>(map.values());


            if(listData.isEmpty()){
                res.setMessage("Terdapat kesalahan, Data Tidak Dapat Ditemukan");
                res.setStatus(0);
                res.setData(null);
            }else{
                res.setStatus(1);
                res.setMessage(null);
                res.setData(listDataDistinct);
            }

            return ResponseEntity.status(HttpStatus.OK).body(res);

        } catch (Exception ex) {
            Logger.getLogger(DGIUnpaidHLORESTController.class.getName()).log(Level.SEVERE, null, ex);
            ResponseErrorDTO res =  new ResponseErrorDTO();
            res.setMessage(ex.getMessage());
//            res.setStatus(HttpStatus.BAD_REQUEST.value());
            res.setStatus(0);
            res.setError(HttpStatus.BAD_REQUEST.getReasonPhrase());
            return ResponseEntity.badRequest().body(res);
        }
    }
}
