package com.eksad.masterdata.service;

import com.eksad.ddms.common.dto.GatewayDPHLODTO;
import com.eksad.ddms.common.dto.GatewayOrafinDTO;
import com.eksad.ddms.common.dto.ResponseOrafinDTO;
import com.eksad.ddms.common.h2.dto.apigateway.ManageWorkOrderParamDTO;
import com.eksad.ddms.common.h2.dto.apigateway.UnpaidDPHLOPullHeaderResponseDTO;
import com.eksad.ddms.common.h2.dto.apigateway.UnpaidDPHLOPullParamDTO;
import com.eksad.ddms.common.h2.dto.apigateway.request.PullNotaJBSCRequestDTO;
import com.eksad.ddms.common.h3.dto.dphlopull.DPHLOPullParamDTO;
import com.eksad.ddms.common.h3.dto.dphlopull.ResponseDPHLOPullDTO;
import com.eksad.ddms.common.h3.dto.managebillingprocess.ResponseDPHLODTO;
import com.eksad.ddms.common.h3.dto.managepartinbounds.PartsManageInboundParamDTO;
import com.eksad.ddms.common.h3.dto.managepartinbounds.ResponseManagePartsInboundDTO;
import com.eksad.ddms.common.h3.dto.managepartsales.ParamManageSalesDTO;
import com.eksad.ddms.common.h3.dto.managepartsales.ResponseManagePartSalesDTO;
import com.eksad.ddms.common.uri.apigateway.APIGatewayURI;
import com.eksad.ddms.masterdata.h3.dto.dgi.DgiResponseDTO;
import com.eksad.ddms.masterdata.h3.dto.dgi.PartsInvoiceRequestDTO;
import com.eksad.ddms.masterdata.h3.dto.dgi.PartsInvoiceResponseDTO;
import com.eksad.masterdata.common.sample.RequestUtil;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.*;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;

import java.io.IOException;
import java.lang.reflect.Field;
import java.nio.charset.StandardCharsets;
import java.security.MessageDigest;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.Duration;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.util.*;

@Service
public class DGIService {
    DateTimeFormatter dateTimeFormatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss", Locale.getDefault());

    private RestTemplate restTemplate = new RestTemplate();

    @Value("${dam.token.key.path}")
    private String TOKEN;

    @Value("${dam.masterdata.h1}")
    private String md_h1_url;

    @Value("${dam.masterdata.h2}")
    private String md_h2_url;

    @Value("${dam.gigr.h3}")
    private String gigr_h3_url;

    @Value("${dam.custpurch.h3}")
    private String custpurch_h3_url;

    @Value("${dam.portal.daya.path}")
    private String PORTAL_DAM_URL;

    @Value("${dam.portal.daya.inv}")
    private String PORTAL_DAM_INV_URL;

    @Value("${daw.portal.daya.inv}")
    private String PORTAL_DAW_INV_URL;

    @Value("${md.portal.profile}")
    public String MD_PROFILE;

    @Autowired
    private RequestUtil requestUtil;

    public ResponseEntity<Object> postRequestPullNJBNSC(PullNotaJBSCRequestDTO dto) throws ParseException {
        Object data = pullNJBNSC(dto);

        if(data != null){
            return ResponseEntity.status(HttpStatus.OK).body(data);
        }
        return null;
    }

    public ResponseEntity<Object> postWorkOrderOrafin(GatewayOrafinDTO dto) throws IOException {
        Object data = pushNJBNSC(dto);
        return ResponseEntity.status(HttpStatus.OK).body(data);
    }

    public ResponseEntity<Object> postRequestPullManagePartInbound(PartsManageInboundParamDTO dto) throws ParseException, JsonProcessingException {
        Object data = pullManagePartInbound(dto);

        System.out.println("data yang dikirim "+new ObjectMapper().writeValueAsString(dto));

        if(data != null){
            return ResponseEntity.status(HttpStatus.OK).body(data);
        }
        return null;
    }

    public ResponseManagePartsInboundDTO postRequestPullManagePartInbound_v1_3(PartsManageInboundParamDTO dto) throws ParseException{
        ResponseManagePartsInboundDTO data = pullManagePartInbound(dto);

        if(data != null){
            return data;
        }
        return null;
    }

    public ResponseEntity<Object> postRequestPullManageWorkOrder(ManageWorkOrderParamDTO dto) throws ParseException{
        Object data = pullManageWorkOrder(dto);

        if(data != null){
            return ResponseEntity.status(HttpStatus.OK).body(data);
        }
        return null;
    }

    public ResponseEntity<Object> postRequestPullManagePartSales(ParamManageSalesDTO dto) throws ParseException{
        Object data = pullManagePartSales(dto);

        if(data != null){
            return ResponseEntity.status(HttpStatus.OK).body(data);
        }
        return null;
    }

    public ResponseManagePartSalesDTO postRequestPullManagePartSales_v1_3(ParamManageSalesDTO dto) throws ParseException{
        ResponseManagePartSalesDTO data = pullManagePartSales(dto);

        if(data != null){
            return data;
        }
        return null;
    }

    public ResponseEntity<Object> postRequestDPHLO(DPHLOPullParamDTO dto) throws ParseException{
        Object data = pullDPHLO(dto);

        if(data != null){
            return ResponseEntity.status(HttpStatus.OK).body(data);
        }
        return null;
    }

    public ResponseEntity<UnpaidDPHLOPullHeaderResponseDTO> postRequestPullUnpaidHLO(UnpaidDPHLOPullParamDTO dto) throws ParseException{
        UnpaidDPHLOPullHeaderResponseDTO data = pullUnpaidHLO(dto);

        if(data != null){
            return ResponseEntity.status(HttpStatus.OK).body(data);
        }
        return null;
    }

    public ResponseEntity<Object> postRequestPullDPHLO(GatewayDPHLODTO dto) throws ParseException{
        Object data = pushDPHLO(dto);

        if(data != null){
            return ResponseEntity.status(HttpStatus.OK).body(data);
        }
        return null;
    }

    public  ResponseDPHLOPullDTO postRequestDPHLO_v1_3(DPHLOPullParamDTO dto) throws ParseException{
        ResponseDPHLOPullDTO data = pullDPHLO(dto);

        if(data != null){
            return data;
        }
        return null;
    }

    public PartsInvoiceResponseDTO postPortalDamParts(PartsInvoiceRequestDTO request) {
        try {
//            String PORTAL_API_KEY = "";

            HashMap<String, String> tokenTimestampMap = getTokenAndRequestTimePortal();
            RestTemplate restTemplate = new RestTemplate();
            HttpHeaders header = new HttpHeaders();
            header.set("Content-Type", "application/json");
            header.set("Accept", "*/*");
            header.set("X-Request-Time", tokenTimestampMap.get("timestamp"));

            ObjectMapper map = new ObjectMapper();
            System.out.println(">>>>> URL DAM : " + PORTAL_DAM_INV_URL);
            System.out.println(">>>>> URL DAW : " + PORTAL_DAW_INV_URL);
            System.out.println(">>>>> PROFILE : " + MD_PROFILE);
            System.out.println(">>>>> request : " + map.writeValueAsString(request));
            System.out.println(">>>>> headers : " + map.writeValueAsString(header));

            if(MD_PROFILE.equalsIgnoreCase("DAM")){
                ResponseEntity<PartsInvoiceResponseDTO> response = restTemplate.exchange(
                        PORTAL_DAM_INV_URL,
                        HttpMethod.POST,
                        new HttpEntity<>(request, header),
                        PartsInvoiceResponseDTO.class);
                if (response.getBody() != null && response.getBody().getData()!=null) {
                    PartsInvoiceResponseDTO res = response.getBody();
                    System.out.println(">>>>> Response portal dam parts : " + map.writeValueAsString(res));
                    if (res.getStatus().equals(1)) res.setMessage(null);
                    if (res.getStatus().equals(0)) res.setStatus(1);
                    return res;
                } else {
                    ResponseEntity<PartsInvoiceResponseDTO> responseDAW = restTemplate.exchange(
                            PORTAL_DAW_INV_URL,
                            HttpMethod.POST,
                            new HttpEntity<>(request, header),
                            PartsInvoiceResponseDTO.class);
                    if (responseDAW.getBody() != null) {
                        PartsInvoiceResponseDTO res = responseDAW.getBody();
                        System.out.println(">>>>> Response portal daw parts : " + map.writeValueAsString(res));
                        if (res.getStatus().equals(1)) res.setMessage(null);
                        if (res.getStatus().equals(0)) res.setStatus(1);
                        return res;
                    }
                }
            }else {
                ResponseEntity<PartsInvoiceResponseDTO> responseDAW = restTemplate.exchange(
                        PORTAL_DAW_INV_URL,
                        HttpMethod.POST,
                        new HttpEntity<>(request, header),
                        PartsInvoiceResponseDTO.class);
                if (responseDAW.getBody() != null) {
                    PartsInvoiceResponseDTO res = responseDAW.getBody();
                    System.out.println(">>>>> Response portal daw parts : " + map.writeValueAsString(res));
                    if (res.getStatus().equals(1)) res.setMessage(null);
                    if (res.getStatus().equals(0)) res.setStatus(1);
                    return res;
                }
            }
            return new PartsInvoiceResponseDTO();
        } catch (Exception e) {
            e.printStackTrace();
            return new PartsInvoiceResponseDTO();
        }
    }

    private HashMap getTokenAndRequestTimePortal() {
        HashMap res = new HashMap();
        String reqTime = System.currentTimeMillis() + "";
        String PORTAL_API_KEY = "";
        String PORTAL_SECRET_KEY = "";
        String token = encryptSHA256(PORTAL_API_KEY + PORTAL_SECRET_KEY + reqTime);
        res.put("timestamp", reqTime);
        res.put("token", token);
        return res;
    }

    public DgiResponseDTO dgiv1_3Validator(String ahmId, String dealerGroupCode) {
        try {
            HttpHeaders httpHeaders = new HttpHeaders();
            httpHeaders.setContentType(MediaType.APPLICATION_JSON);
            httpHeaders.setAccept(Arrays.asList(MediaType.APPLICATION_JSON));
            httpHeaders.add("Authorization", TOKEN);

            RestTemplate restTemplate = new RestTemplate();

            Map<String, String> req = new HashMap<>();

            req.put("dealerAhmCode", ahmId);
            req.put("dealerMaster", dealerGroupCode);

            System.out.println("h1_url "+md_h1_url);

            RequestUtil requestUtil = new RequestUtil();

            ResponseEntity<DgiResponseDTO> res =  restTemplate.exchange(
                    md_h1_url +
                            "/validate.dealer.group",
                    HttpMethod.POST,
                    new HttpEntity<>(new ObjectMapper().writeValueAsString(req), httpHeaders),
                    DgiResponseDTO.class
            );


            DgiResponseDTO response = res.getBody();

            System.out.println("AWWW" +new ObjectMapper().writeValueAsString(response));
            return response;

        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    public void checkToTimeIs7Days(String fromTime, String toTime) throws Exception {
        DateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        Date from = dateFormat.parse(fromTime);
        Date to = dateFormat.parse(toTime);
        LocalDateTime fromLocal = from.toInstant().atZone(ZoneId.of("Asia/Jakarta")).toLocalDateTime();
        LocalDateTime toLocal = to.toInstant().atZone(ZoneId.of("Asia/Jakarta")).toLocalDateTime();
        Duration durasi = Duration.between(fromLocal, toLocal);
        long dayDifference = Math.abs(durasi.toHours());

        if (dayDifference > (7 * 24) + 1) {  // if duration greater than 24*7 + 1 hours
            throw new Exception("tenggat waktu hanya diperbolehkan 7 hari");
        }
    }

    public void nullEmptyExceptionHelper(Object item, List<String> pass) throws Exception {
        for (Field field : item.getClass().getDeclaredFields()) {
            field.setAccessible(true);
            System.out.println(field.getName());
            if (Collection.class.isAssignableFrom(field.getType())) {
                continue;
            } else if (pass.contains(field.getName().trim())) {
                continue;
            } else if (field.get(item) == null || String.valueOf(field.get(item)).trim().isEmpty()) {
                throw new Exception(field.getName() + " tidak boleh kosong");
            }
        }
    }

    public void exceptionHelper(String time, String format, String message) throws Exception {
        if (time == null || time.trim().isEmpty()) {
            throw new Exception(message + " tidak boleh kosong");
        }

        try {
            SimpleDateFormat dateFormat = new SimpleDateFormat(format);
            dateFormat.parse(time);
        } catch (ParseException e) {
            throw new Exception("format " + message.concat(" tidak sesuai, seharusnya ").concat(format));
        }

    }

    public void exceptionHelperNotMandatory(String time, String format, String message) throws Exception {
        if (time != null && !time.trim().isEmpty()) {
            try {
                SimpleDateFormat dateFormat = new SimpleDateFormat(format);
                dateFormat.parse(time);
            } catch (ParseException e) {
                throw new Exception("format " + message.concat(" tidak sesuai, seharusnya ").concat(format));
            }
        }
    }

    public boolean isNotValidDate(String input) {
        try {
            dateTimeFormatter.parse(input);
            return false;
        } catch (Exception e) {
            return true;
        }
    }

    public void dealerValidation(String dealerHeader, String dealerRequest) throws Exception {
        if (dealerHeader != null && dealerHeader != dealerRequest) {
            if (!dealerHeader.equalsIgnoreCase(dealerRequest)) {
                throw new Exception("gagal dealerId " + dealerRequest + " tidak sesuai");
            }
        }
    }

    private String encryptSHA256(String originalString) {
        try {
            final MessageDigest digest = MessageDigest.getInstance("SHA-256");
            final byte[] hashbytes = digest.digest(originalString.getBytes(StandardCharsets.UTF_8));
            return bytesToHex(hashbytes);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return "";
    }

    private static String bytesToHex(byte[] hash) {
        StringBuilder hexString = new StringBuilder(2 * hash.length);
        for (int i = 0; i < hash.length; i++) {
            String hex = Integer.toHexString(0xff & hash[i]);
            if (hex.length() == 1) {
                hexString.append('0');
            }
            hexString.append(hex);
        }
        return hexString.toString();
    }

    private Object pullManageWorkOrder(ManageWorkOrderParamDTO dto) {
        ResponseEntity<Object> response = restTemplate.exchange(
                md_h2_url
                        + APIGatewayURI.PULL_MANAGE_WORKORDER.getUri(),
                HttpMethod.POST,
                new HttpEntity<>(dto, requestUtil.getPreFormattedRequestWithToken().getHeaders()),
                Object.class);
        return response.getBody();
    }

    private Object pushNJBNSC(GatewayOrafinDTO dto) throws IOException {
        ObjectMapper obm = new ObjectMapper();
        ResponseEntity<String> response = restTemplate.exchange(
                md_h2_url
                        + APIGatewayURI.PUSH_NSC_NJB.getUri(),
                HttpMethod.POST,
                new HttpEntity<>(dto, requestUtil.getPreFormattedRequestWithToken().getHeaders()),
                String.class);
        ResponseOrafinDTO resOra = obm.readValue(response.getBody(), new TypeReference<ResponseOrafinDTO>() { });
        return resOra;
    }

    private ResponseManagePartsInboundDTO pullManagePartInbound(PartsManageInboundParamDTO dto) {
        ResponseEntity<ResponseManagePartsInboundDTO> response = restTemplate.exchange(
                gigr_h3_url
                        + APIGatewayURI.PULL_MANAGE_PART_INBOUND.getUri(),
                HttpMethod.POST,
                new HttpEntity<>(dto, requestUtil.getPreFormattedRequestWithToken().getHeaders()),
                ResponseManagePartsInboundDTO.class);
        return response.getBody();
    }

    private Object pullNJBNSC(PullNotaJBSCRequestDTO dto) {
        ResponseEntity<Object> response = restTemplate.exchange(
                md_h2_url
                        + APIGatewayURI.PULL_NSC_NJB.getUri(),
                HttpMethod.POST,
                new HttpEntity<>(dto, requestUtil.getPreFormattedRequestWithToken().getHeaders()),
                Object.class);
        return response.getBody();
    }

    private ResponseManagePartSalesDTO pullManagePartSales(ParamManageSalesDTO dto) {
        ResponseEntity<ResponseManagePartSalesDTO> response = restTemplate.exchange(
                custpurch_h3_url
                        + APIGatewayURI.PULL_MANAGE_PARTSALES.getUri(),
                HttpMethod.POST,
                new HttpEntity<>(dto, requestUtil.getPreFormattedRequestWithToken().getHeaders()),
                ResponseManagePartSalesDTO.class);
        return response.getBody();
    }

    private ResponseDPHLOPullDTO pullDPHLO(DPHLOPullParamDTO dto) {
        ResponseEntity<ResponseDPHLOPullDTO> response = restTemplate.exchange(
                custpurch_h3_url
                        + APIGatewayURI.PULL_DP_HLO.getUri(),
                HttpMethod.POST,
                new HttpEntity<>(dto, requestUtil.getPreFormattedRequestWithToken().getHeaders()),
                ResponseDPHLOPullDTO.class);
        return response.getBody();
    }

    private UnpaidDPHLOPullHeaderResponseDTO pullUnpaidHLO(UnpaidDPHLOPullParamDTO dto) {
        ResponseEntity<UnpaidDPHLOPullHeaderResponseDTO> response = restTemplate.exchange(
                custpurch_h3_url
                        + APIGatewayURI.PULL_UNPAID_HLO.getUri(),
                HttpMethod.POST,
                new HttpEntity<>(dto, requestUtil.getPreFormattedRequestWithToken().getHeaders()),
                UnpaidDPHLOPullHeaderResponseDTO.class);
        return response.getBody();
    }

    public Object pushDPHLO(GatewayDPHLODTO dto) {
        ResponseEntity<ResponseDPHLODTO> response = restTemplate.exchange(
                custpurch_h3_url
                        + APIGatewayURI.PUSH_DP_HLO.getUri(),
                HttpMethod.POST,
                new HttpEntity<>(dto, requestUtil.getPreFormattedRequestWithToken().getHeaders()),
                ResponseDPHLODTO.class);
        return response.getBody();
    }
}
